package be.iminds.ilabt.jfed.testing.tests.highlevel;

import be.iminds.ilabt.jfed.testing.base.ApiTestConfig;
import be.iminds.ilabt.jfed.testing.base.ApiTestConfigProperty;
import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GuiLogicTestConfig implements ApiTestConfig {
    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @Nonnull private final Proxy amConnectionProxy;
    @Nonnull private final String amVersion;
    @Nonnull private final SSHKeys sshKeys;
    @Nonnull private final List<Resource> resources;
    @Nonnull private final AnsibleTest ansibleTest;
    @Nonnull private final NodeLoginTest nodeLoginTest;

    public GuiLogicTestConfig() {
        this.amConnectionProxy = new Proxy();
        this.amVersion = "auto";
        this.sshKeys = new SSHKeys();
        this.resources = Collections.singletonList(new Resource());
        this.ansibleTest = new AnsibleTest();
        this.nodeLoginTest = new NodeLoginTest();
    }

    @JsonCreator
    public GuiLogicTestConfig(@Nullable @JsonProperty("amConnectionProxy") Proxy amConnectionProxy,
                              @Nullable @JsonProperty("amVersion") String amVersion,
                              @Nullable @JsonProperty("sshKeys") SSHKeys sshKeys,
                              @Nullable @JsonProperty("resources") List<Resource> resources,
                              @Nullable @JsonProperty("ansibleTest") AnsibleTest ansibleTest,
                              @Nullable @JsonProperty("nodeLoginTest") NodeLoginTest nodeLoginTest) {
        this.amConnectionProxy = amConnectionProxy == null ? new GuiLogicTestConfig().getAmConnectionProxy() : amConnectionProxy;
        this.amVersion = amVersion == null ? new GuiLogicTestConfig().getAmVersion() : amVersion;
        this.sshKeys = sshKeys == null ? new GuiLogicTestConfig().getSshKeys() : sshKeys;
        this.resources = resources == null ? new GuiLogicTestConfig().getResources() : resources;
        this.ansibleTest = ansibleTest == null ? new GuiLogicTestConfig().getAnsibleTest() : ansibleTest;
        this.nodeLoginTest = nodeLoginTest == null ? new GuiLogicTestConfig().getNodeLoginTest() : nodeLoginTest;
    }

    @Override
    @JsonIgnore
    public boolean isValid() {
        return true; //TODO
    }

    @JsonProperty
    @Nonnull
    public Proxy getAmConnectionProxy() {
        return amConnectionProxy;
    }

    @JsonProperty
    @Nonnull
    public String getAmVersion() {
        return amVersion;
    }

    @JsonProperty
    @Nonnull
    public SSHKeys getSshKeys() {
        return sshKeys;
    }

    @JsonProperty
    @Nonnull
    public List<Resource> getResources() {
        return resources;
    }

    @JsonProperty
    @Nonnull
    public AnsibleTest getAnsibleTest() {
        return ansibleTest;
    }

    @JsonProperty
    @Nonnull
    public NodeLoginTest getNodeLoginTest() {
        return nodeLoginTest;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof GuiLogicTestConfig)) return false;

        GuiLogicTestConfig that = (GuiLogicTestConfig) o;

        if (!amConnectionProxy.equals(that.amConnectionProxy)) return false;
        if (!amVersion.equals(that.amVersion)) return false;
        if (!sshKeys.equals(that.sshKeys)) return false;
        if (!resources.equals(that.resources)) return false;
        if (!ansibleTest.equals(that.ansibleTest)) return false;
        return nodeLoginTest.equals(that.nodeLoginTest);
    }

    @Override
    public int hashCode() {
        int result = amConnectionProxy.hashCode();
        result = 31 * result + amVersion.hashCode();
        result = 31 * result + sshKeys.hashCode();
        result = 31 * result + resources.hashCode();
        result = 31 * result + ansibleTest.hashCode();
        result = 31 * result + nodeLoginTest.hashCode();
        return result;
    }

    @Nonnull
    @Override
    @JsonIgnore
    public String stringValue() {
        try {
            return MAPPER.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to covert "+getClass().getName()+" to String", e);
        }
    }

    @Override
    public String toString() {
        return stringValue();
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class AnsibleTest {
        @Nonnull private final Boolean enabled;
        @Nonnull private final Boolean debug;
        @Nonnull private final String playbookExe;
        @Nullable private final String cookbookUrl;
        @Nullable private final String cookbookContent;
        @Nonnull private final String failureRegex;
        @Nonnull private final String warningRegex;
        @Nonnull private final String successRegex;
        @Nonnull private final Proxy proxy;

        public AnsibleTest() {
            this.enabled = false;
            this.proxy = new Proxy();
            this.debug = false;
            this.playbookExe = "/usr/local/bin/ansible-playbook";
            this.cookbookUrl = null;
            this.cookbookContent = null;
            this.failureRegex = "failed=[^0]";
            this.warningRegex = "unreachable=[^0]";
            this.successRegex = "ok=[^0]";
        }

        @JsonCreator
        public AnsibleTest(@Nullable @JsonProperty("enabled") Boolean enabled,
                           @Nullable @JsonProperty("proxy") Proxy proxy,
                           @Nullable @JsonProperty("debug") Boolean debug,
                           @Nullable @JsonProperty("playbookExe") String playbookExe,
                           @Nullable @JsonProperty("cookbookUrl") String cookbookUrl,
                           @Nullable @JsonProperty("cookbookContent") String cookbookContent,
                           @Nullable @JsonProperty("failureRegex") String failureRegex,
                           @Nullable @JsonProperty("warningRegex") String warningRegex,
                           @Nullable @JsonProperty("successRegex") String successRegex) {
            this.enabled = enabled == null ? new AnsibleTest().isEnabled() : enabled;
            this.proxy = proxy == null ? new AnsibleTest().getProxy() : proxy;
            this.debug = debug == null ? new AnsibleTest().getDebug() : debug;
            this.playbookExe = playbookExe == null ? new AnsibleTest().getPlaybookExe() : playbookExe;
            this.cookbookUrl = cookbookUrl;
            this.cookbookContent = cookbookContent;
            this.failureRegex = failureRegex == null ? new AnsibleTest().getFailureRegex() : failureRegex;
            this.warningRegex = warningRegex == null ? new AnsibleTest().getWarningRegex() : warningRegex;
            this.successRegex = successRegex == null ? new AnsibleTest().getSuccessRegex() : successRegex;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="enables the runAnsible() step of the test")
        public boolean isEnabled() {
            return enabled;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="(SSH) proxy to use for the ansible tests")
        @Nonnull
        public Proxy getProxy() {
            return proxy;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="sets the debug flag when calling ansible")
        public boolean getDebug() {
            return debug;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="full filename of the ansible-playbook executable")
        @Nonnull
        public String getPlaybookExe() {
            return playbookExe;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="URL of the cookbook to execute (either cookbookUrl or cookbookUrl must be provided, and not both)")
        @Nullable
        public String getCookbookUrl() {
            return cookbookUrl;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="Content of the cookbook to execute (either cookbookUrl or cookbookUrl must be provided, and not both)")
        @Nullable
        public String getCookbookContent() {
            return cookbookContent;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="When this regex matches a line in the ansible output, the test state is considered failed.")
        @Nonnull
        public String getFailureRegex() {
            return failureRegex;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="When this regex matches a line in the ansible output, the test state is considered warning. (Unless it is failed.)")
        @Nonnull
        public String getWarningRegex() {
            return warningRegex;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="When this regex matches a line in the ansible output, the test state is considered succes. (Unless it is failed or warning.)")
        @Nonnull
        public String getSuccessRegex() {
            return successRegex;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof AnsibleTest)) return false;

            AnsibleTest that = (AnsibleTest) o;

            if (!enabled.equals(that.enabled)) return false;
            if (!debug.equals(that.debug)) return false;
            if (!playbookExe.equals(that.playbookExe)) return false;
            if (cookbookUrl != null ? !cookbookUrl.equals(that.cookbookUrl) : that.cookbookUrl != null) return false;
            if (cookbookContent != null ? !cookbookContent.equals(that.cookbookContent) : that.cookbookContent != null)
                return false;
            if (!failureRegex.equals(that.failureRegex)) return false;
            if (!warningRegex.equals(that.warningRegex)) return false;
            if (!successRegex.equals(that.successRegex)) return false;
            return proxy.equals(that.proxy);
        }

        @Override
        public int hashCode() {
            int result = enabled.hashCode();
            result = 31 * result + debug.hashCode();
            result = 31 * result + playbookExe.hashCode();
            result = 31 * result + (cookbookUrl != null ? cookbookUrl.hashCode() : 0);
            result = 31 * result + (cookbookContent != null ? cookbookContent.hashCode() : 0);
            result = 31 * result + failureRegex.hashCode();
            result = 31 * result + warningRegex.hashCode();
            result = 31 * result + successRegex.hashCode();
            result = 31 * result + proxy.hashCode();
            return result;
        }
    }

    public enum ProxyMode { OFF, MANUAL, USER_AUTHORITY };
    public enum ProxyKeySource { USER, NODE };
    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class Proxy {
        @Nonnull private final ProxyMode mode; //valid option: off (default), manual(use preferences below), user(automatically use proxy of user authority)

        @Nullable private final String hostName;
        @Nullable private final String hostKey;
        private final int hostPort;
        @Nonnull private final String username; //default: same username as logged in user (same as "$USER"). Some limited variable substitution is done:
        // - $NODE: username for target node
        // - $USER: logged in user name (last part of user URN)
        // - $SLICE: slice name (last part of slice URN)
        @Nonnull private final ProxyKeySource keySource; //valid choices:
        // - user (take SSH keys from user login cert and key)
        // - node (use the same SSH keys as the one to login to the actual target node, these are typically generated by NodeLoginTester, unless fixed_ssh_public_key_file says otherwise)
        // - file: take from file name specified in sshProxyPrivateKeyFile and sshProxyPublicKeyFile option (not yet implemented)


        public Proxy() {
            this.mode = ProxyMode.OFF;
            this.hostName = null;
            this.hostKey = null;
            this.hostPort = 22;
            this.username = "$USER";
            this.keySource = ProxyKeySource.USER;
        }

        @JsonCreator
        public Proxy(@Nullable @JsonProperty("mode") ProxyMode mode,
                     @Nullable @JsonProperty("hostName") String hostName,
                     @Nullable @JsonProperty("hostKey") String hostKey,
                     @Nullable @JsonProperty("hostPort") Integer hostPort,
                     @Nullable @JsonProperty("username") String username,
                     @Nullable @JsonProperty("keySource") ProxyKeySource keySource) {
            this.mode = mode == null ? new Proxy().getMode() : mode;
            this.hostName = hostName;
            this.hostKey = hostKey;
            this.hostPort = hostPort == null ? new Proxy().getHostPort() : hostPort;
            this.username = username == null ? new Proxy().getUsername() : username;
            this.keySource = keySource == null ? new Proxy().getKeySource() : keySource;
        }

        @JsonProperty
        @Nonnull
        public ProxyMode getMode() {
            return mode;
        }

        @JsonProperty
        @Nullable
        public String getHostName() {
            return hostName;
        }

        @JsonProperty
        @Nullable
        public String getHostKey() {
            return hostKey;
        }

        @JsonProperty
        public int getHostPort() {
            return hostPort;
        }

        @JsonProperty
        @Nonnull
        public String getUsername() {
            return username;
        }

        @JsonProperty
        @Nonnull
        public ProxyKeySource getKeySource() {
            return keySource;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof Proxy)) return false;

            Proxy proxy = (Proxy) o;

            if (hostPort != proxy.hostPort) return false;
            if (mode != proxy.mode) return false;
            if (hostName != null ? !hostName.equals(proxy.hostName) : proxy.hostName != null) return false;
            if (hostKey != null ? !hostKey.equals(proxy.hostKey) : proxy.hostKey != null) return false;
            if (!username.equals(proxy.username)) return false;
            return keySource == proxy.keySource;
        }

        @Override
        public int hashCode() {
            int result = mode.hashCode();
            result = 31 * result + (hostName != null ? hostName.hashCode() : 0);
            result = 31 * result + (hostKey != null ? hostKey.hashCode() : 0);
            result = 31 * result + hostPort;
            result = 31 * result + username.hashCode();
            result = 31 * result + keySource.hashCode();
            return result;
        }
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class NodeLoginTest {
        private final boolean enabled;
        private final long deadlineSeconds;
        @Nullable private final String preferredUser;
        @Nullable private final Integer forceIpVersion;
        private final boolean useExternalSsh;
        @Nonnull private final Proxy proxy;

        public NodeLoginTest() {
            this.enabled = true;
            this.deadlineSeconds = 20 * 60;
            this.preferredUser = null;
            this.forceIpVersion = null;
            this.useExternalSsh = false;
            this.proxy = new Proxy();
        }

        @JsonCreator
        public NodeLoginTest(@Nullable @JsonProperty("enabled") Boolean enabled,
                             @Nullable @JsonProperty("deadlineSeconds") Long deadlineSeconds,
                             @Nullable @JsonProperty("preferredUser") String preferredUser,
                             @Nullable @JsonProperty("forceIpVersion") Integer forceIpVersion,
                             @Nullable @JsonProperty("useExternalSsh") Boolean useExternalSsh,
                             @Nullable @JsonProperty("proxy") Proxy proxy) {
            this.enabled = enabled == null ? new NodeLoginTest().isEnabled() : enabled;
            this.deadlineSeconds = deadlineSeconds == null ? new NodeLoginTest().getDeadlineSeconds() : deadlineSeconds;
            this.preferredUser = preferredUser;
            this.forceIpVersion = forceIpVersion;
            this.useExternalSsh = useExternalSsh == null ? new NodeLoginTest().getUseExternalSsh() : useExternalSsh;
            this.proxy = proxy == null ? new NodeLoginTest().getProxy() : proxy;
        }

        @JsonProperty
        public boolean isEnabled() {
            return enabled;
        }

        @JsonProperty
        public long getDeadlineSeconds() {
            return deadlineSeconds;
        }

        @JsonProperty
        @Nullable
        public String getPreferredUser() {
            return preferredUser;
        }

        @JsonProperty
        @Nullable
        public Integer getForceIpVersion() {
            return forceIpVersion;
        }

        @JsonProperty
        public boolean getUseExternalSsh() {
            return useExternalSsh;
        }

        @JsonProperty
        @Nonnull
        public Proxy getProxy() {
            return proxy;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof NodeLoginTest)) return false;

            NodeLoginTest that = (NodeLoginTest) o;

            if (enabled != that.enabled) return false;
            if (deadlineSeconds != that.deadlineSeconds) return false;
            if (useExternalSsh != that.useExternalSsh) return false;
            if (preferredUser != null ? !preferredUser.equals(that.preferredUser) : that.preferredUser != null)
                return false;
            if (forceIpVersion != null ? !forceIpVersion.equals(that.forceIpVersion) : that.forceIpVersion != null)
                return false;
            return proxy.equals(that.proxy);
        }

        @Override
        public int hashCode() {
            int result = (enabled ? 1 : 0);
            result = 31 * result + (int) (deadlineSeconds ^ (deadlineSeconds >>> 32));
            result = 31 * result + (preferredUser != null ? preferredUser.hashCode() : 0);
            result = 31 * result + (forceIpVersion != null ? forceIpVersion.hashCode() : 0);
            result = 31 * result + (useExternalSsh ? 1 : 0);
            result = 31 * result + proxy.hashCode();
            return result;
        }
    }

    public enum SSHKeySource { TEST_USER, RANDOM };

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class SSHKeys {
        @Nonnull private final SSHKeySource source;

        //maybe future, but not sure how to do this securily from fedmon:
        // , fixedContent, fixedFile
//        @Nonnull private final String publicKeyContent;
//        @Nonnull private final String privateKeyContent;
//        @Nonnull private final String publicKeyFile;
//        @Nonnull private final String privateKeyFile;
//        @Nonnull private final String privateKeyPassword;


        public SSHKeys() {
            this.source = SSHKeySource.RANDOM;
        }

        @JsonCreator
        public SSHKeys(@Nullable @JsonProperty("source") SSHKeySource source) {
            this.source = source == null ? new SSHKeys().getSource() : source;
        }

        @JsonProperty
        @Nonnull
        public SSHKeySource getSource() {
            return source;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof SSHKeys)) return false;

            SSHKeys sshKeys = (SSHKeys) o;

            return source == sshKeys.source;
        }

        @Override
        public int hashCode() {
            return source.hashCode();
        }
    }

    public enum RequestRSpecSource { PROVIDE_URL, PROVIDE_FILE, PROVIDE_CONTENT, GENERATE_USING_JFED, GENERATE_RAW, ALREADY_PROVISIONED };
    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class RequestRSpec {
        @Nonnull private final RequestRSpecSource source;
        @Nullable private final String providedContentSource; //url, file or content as specified in source

        //If RSpec content is provided (URL, file or direct)
        private final boolean assignTestedServerToUnassignedNodes;

        //If RSpec is generated
        private final int nodeCount;

        //If RSpec is generated by jFed
        @Nullable private final String jFedResourceClass; //if null, a class will be picked automatically, using the order in the jFed GUI (top being highest priority)

        //If RSpec is generated "raw"
        @Nonnull private final String nodeSliverType;
        private final boolean nodeExclusive;
        @Nonnull private final List<String> nodeComponentUrn;

        //used for all methods
        private final boolean autoAssignSpecificNodesUsingListResources;
        private final boolean useNitosLease;

        public RequestRSpec() {
            this.source = RequestRSpecSource.GENERATE_USING_JFED;
            this.providedContentSource = null;
            this.assignTestedServerToUnassignedNodes = true;
            this.nodeCount = 1;
            this.jFedResourceClass = null;
            this.nodeSliverType = "raw-pc";
            this.nodeExclusive = true;
            this.nodeComponentUrn = Collections.emptyList();
            this.autoAssignSpecificNodesUsingListResources = false;
            this.useNitosLease = false;
        }

        @JsonCreator
        public RequestRSpec(@Nullable @JsonProperty("source") RequestRSpecSource source,
                            @Nullable @JsonProperty("providedContentSource") String providedContentSource,
                            @Nullable @JsonProperty("assignTestedServerToUnassignedNodes") Boolean assignTestedServerToUnassignedNodes,
                            @Nullable @JsonProperty("nodeCount") Integer nodeCount,
                            @Nullable @JsonProperty("jFedResourceClass") String jFedResourceClass,
                            @Nullable @JsonProperty("nodeSliverType") String nodeSliverType,
                            @Nullable @JsonProperty("nodeExclusive") Boolean nodeExclusive,
                            @Nullable @JsonProperty("nodeComponentUrn") List<String> nodeComponentUrn,
                            @Nullable @JsonProperty("autoAssignSpecificNodesUsingListResources") Boolean autoAssignSpecificNodesUsingListResources,
                            @Nullable @JsonProperty("useNitosLease") Boolean useNitosLease) {
            this.source = source == null ? new RequestRSpec().getSource() : source;
            this.providedContentSource = providedContentSource;
            this.assignTestedServerToUnassignedNodes = assignTestedServerToUnassignedNodes == null ? new RequestRSpec().getAssignTestedServerToUnassignedNodes() : assignTestedServerToUnassignedNodes;;
            this.nodeCount = nodeCount == null ? new RequestRSpec().getNodeCount() : nodeCount;
            this.jFedResourceClass = jFedResourceClass == null ? new RequestRSpec().getNodeSliverType() : jFedResourceClass;
            this.nodeSliverType = nodeSliverType == null ? new RequestRSpec().getNodeSliverType() : nodeSliverType;
            this.nodeExclusive = nodeExclusive == null ? new RequestRSpec().getNodeExclusive() : nodeExclusive;
            this.nodeComponentUrn = nodeComponentUrn == null ? new RequestRSpec().getNodeComponentUrn() : nodeComponentUrn;
            this.autoAssignSpecificNodesUsingListResources = autoAssignSpecificNodesUsingListResources == null ? new RequestRSpec().getAutoAssignSpecificNodesUsingListResources() : autoAssignSpecificNodesUsingListResources;
            this.useNitosLease = useNitosLease == null ? new RequestRSpec().getUseNitosLease() : useNitosLease;
        }

        @JsonProperty
        @Nonnull
        public RequestRSpecSource getSource() {
            return source;
        }

        @JsonProperty
        @Nullable
        public String getProvidedContentSource() {
            return providedContentSource;
        }

        @JsonProperty
        public boolean getAssignTestedServerToUnassignedNodes() {
            return assignTestedServerToUnassignedNodes;
        }

        @JsonProperty
        public int getNodeCount() {
            return nodeCount;
        }

        @JsonProperty
        @Nullable
        public String getjFedResourceClass() {
            return jFedResourceClass;
        }

        @JsonProperty
        @Nonnull
        public String getNodeSliverType() {
            return nodeSliverType;
        }

        @JsonProperty
        public boolean getNodeExclusive() {
            return nodeExclusive;
        }

        @JsonProperty
        @Nonnull
        public List<String> getNodeComponentUrn() {
            return nodeComponentUrn;
        }

        @JsonProperty
        public boolean getAutoAssignSpecificNodesUsingListResources() {
            return autoAssignSpecificNodesUsingListResources;
        }

        @JsonProperty
        public boolean getUseNitosLease() {
            return useNitosLease;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof RequestRSpec)) return false;

            RequestRSpec that = (RequestRSpec) o;

            if (assignTestedServerToUnassignedNodes != that.assignTestedServerToUnassignedNodes) return false;
            if (nodeCount != that.nodeCount) return false;
            if (nodeExclusive != that.nodeExclusive) return false;
            if (autoAssignSpecificNodesUsingListResources != that.autoAssignSpecificNodesUsingListResources)
                return false;
            if (useNitosLease != that.useNitosLease) return false;
            if (source != that.source) return false;
            if (providedContentSource != null ? !providedContentSource.equals(that.providedContentSource) : that.providedContentSource != null)
                return false;
            if (jFedResourceClass != null ? !jFedResourceClass.equals(that.jFedResourceClass) : that.jFedResourceClass != null)
                return false;
            if (!nodeSliverType.equals(that.nodeSliverType)) return false;
            return nodeComponentUrn.equals(that.nodeComponentUrn);
        }

        @Override
        public int hashCode() {
            int result = source.hashCode();
            result = 31 * result + (providedContentSource != null ? providedContentSource.hashCode() : 0);
            result = 31 * result + (assignTestedServerToUnassignedNodes ? 1 : 0);
            result = 31 * result + nodeCount;
            result = 31 * result + (jFedResourceClass != null ? jFedResourceClass.hashCode() : 0);
            result = 31 * result + nodeSliverType.hashCode();
            result = 31 * result + (nodeExclusive ? 1 : 0);
            result = 31 * result + nodeComponentUrn.hashCode();
            result = 31 * result + (autoAssignSpecificNodesUsingListResources ? 1 : 0);
            result = 31 * result + (useNitosLease ? 1 : 0);
            return result;
        }
    }

    public enum ProjectSource { NONE, AUTO, PROVIDED }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class Slice {
        @Nullable private final String sliceName;
        private final boolean failOnExistingSlice;
        private final boolean failOnNonExistingSlice;
        private final long expireTimeMin;
        private final boolean renewExistingSliceIfNeeded;

        @Nonnull private final ProjectSource projectSource;
        private final boolean failIfNoProject; //if projectSource is AUTO or PROVIDED
        @Nullable private final String project; //if projectSource is PROVIDED

        public Slice() {
            this.sliceName = null;
            this.failOnExistingSlice = true;
            this.failOnNonExistingSlice = false;
            this.expireTimeMin = 60;
            this.renewExistingSliceIfNeeded = true;
            this.projectSource = ProjectSource.AUTO;
            this.failIfNoProject = false;
            this.project = null;
        }

        @JsonCreator
        public Slice(@Nullable @JsonProperty("sliceName") String sliceName,
                     @Nullable @JsonProperty("failOnExistingSlice") Boolean failOnExistingSlice,
                     @Nullable @JsonProperty("failOnNonExistingSlice") Boolean failOnNonExistingSlice,
                     @Nullable @JsonProperty("expireTimeMin") Long expireTimeMin,
                     @Nullable @JsonProperty("renewExistingSliceIfNeeded") Boolean renewExistingSliceIfNeeded,
                     @Nullable @JsonProperty("projectSource") ProjectSource projectSource,
                     @Nullable @JsonProperty("failIfNoProject") Boolean failIfNoProject,
                     @Nullable @JsonProperty("project") String project) {
            this.sliceName = sliceName == null ? new Slice().getSliceName() : sliceName;
            this.failOnExistingSlice = failOnExistingSlice == null ? new Slice().getFailOnExistingSlice() : failOnExistingSlice;
            this.failOnNonExistingSlice = failOnNonExistingSlice == null ? new Slice().getFailOnNonExistingSlice() : failOnNonExistingSlice;
            this.expireTimeMin = expireTimeMin == null ? new Slice().getExpireTimeMin() : expireTimeMin;
            this.renewExistingSliceIfNeeded = renewExistingSliceIfNeeded == null ? new Slice().getRenewExistingSliceIfNeeded() : renewExistingSliceIfNeeded;
            this.projectSource = projectSource == null ? new Slice().getProjectSource() : projectSource;
            this.failIfNoProject = failIfNoProject == null ? new Slice().getFailIfNoProject() : failIfNoProject;
            this.project = project == null ? new Slice().getProject() : project;
        }

        @Nullable
        public String getSliceName() {
            return sliceName;
        }

        public boolean getFailOnExistingSlice() {
            return failOnExistingSlice;
        }

        public boolean getFailOnNonExistingSlice() {
            return failOnNonExistingSlice;
        }

        public long getExpireTimeMin() {
            return expireTimeMin;
        }

        public boolean getRenewExistingSliceIfNeeded() {
            return renewExistingSliceIfNeeded;
        }

        @Nonnull
        public ProjectSource getProjectSource() {
            return projectSource;
        }

        public boolean getFailIfNoProject() {
            return failIfNoProject;
        }

        @Nullable
        public String getProject() {
            return project;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof Slice)) return false;

            Slice slice = (Slice) o;

            if (failOnExistingSlice != slice.failOnExistingSlice) return false;
            if (failOnNonExistingSlice != slice.failOnNonExistingSlice) return false;
            if (expireTimeMin != slice.expireTimeMin) return false;
            if (renewExistingSliceIfNeeded != slice.renewExistingSliceIfNeeded) return false;
            if (failIfNoProject != slice.failIfNoProject) return false;
            if (sliceName != null ? !sliceName.equals(slice.sliceName) : slice.sliceName != null) return false;
            if (projectSource != slice.projectSource) return false;
            return project != null ? project.equals(slice.project) : slice.project == null;
        }

        @Override
        public int hashCode() {
            int result = sliceName != null ? sliceName.hashCode() : 0;
            result = 31 * result + (failOnExistingSlice ? 1 : 0);
            result = 31 * result + (failOnNonExistingSlice ? 1 : 0);
            result = 31 * result + (int) (expireTimeMin ^ (expireTimeMin >>> 32));
            result = 31 * result + (renewExistingSliceIfNeeded ? 1 : 0);
            result = 31 * result + projectSource.hashCode();
            result = 31 * result + (failIfNoProject ? 1 : 0);
            result = 31 * result + (project != null ? project.hashCode() : 0);
            return result;
        }
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class Provision {
        //Allocate, Provision, POA    (skipped if requestRspec method is "already_povisioned")
        private final long provisionExpireTimeMin;
        private final boolean useGeniEndTimeOptionForCreateSliver;

        public Provision() {
            this.provisionExpireTimeMin = 25;
            this.useGeniEndTimeOptionForCreateSliver = false;
        }
        @JsonCreator
        public Provision(@Nullable @JsonProperty("provisionExpireTimeMin") Long provisionExpireTimeMin,
                         @Nullable @JsonProperty("useGeniEndTimeOptionForCreateSliver") Boolean useGeniEndTimeOptionForCreateSliver) {
            this.provisionExpireTimeMin = provisionExpireTimeMin == null ? new Provision().getProvisionExpireTimeMin() : provisionExpireTimeMin;
            this.useGeniEndTimeOptionForCreateSliver = useGeniEndTimeOptionForCreateSliver == null ? new Provision().getUseGeniEndTimeOptionForCreateSliver() : useGeniEndTimeOptionForCreateSliver;
        }

        @JsonProperty
        public long getProvisionExpireTimeMin() {
            return provisionExpireTimeMin;
        }

        @JsonProperty
        public boolean getUseGeniEndTimeOptionForCreateSliver() {
            return useGeniEndTimeOptionForCreateSliver;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof Provision)) return false;

            Provision provision = (Provision) o;

            if (provisionExpireTimeMin != provision.provisionExpireTimeMin) return false;
            return useGeniEndTimeOptionForCreateSliver == provision.useGeniEndTimeOptionForCreateSliver;
        }

        @Override
        public int hashCode() {
            int result = (int) (provisionExpireTimeMin ^ (provisionExpireTimeMin >>> 32));
            result = 31 * result + (useGeniEndTimeOptionForCreateSliver ? 1 : 0);
            return result;
        }
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class WaitForReady {
        private final long maxTimeMin;
        private final boolean skipFinalManifestCheck;
        private final boolean requireNodesInManifest;
        private final long extraWaitTimeSeconds;

        public WaitForReady() {
            this.maxTimeMin = 20;
            this.skipFinalManifestCheck = false;
            this.requireNodesInManifest = true;
            this.extraWaitTimeSeconds = 0;
        }
        @JsonCreator
        public WaitForReady(@Nullable @JsonProperty("maxTimeMin") Long maxTimeMin,
                            @Nullable @JsonProperty("skipFinalManifestCheck") Boolean skipFinalManifestCheck,
                            @Nullable @JsonProperty("requireNodesInManifest") Boolean requireNodesInManifest,
                            @Nullable @JsonProperty("extraWaitTimeSeconds") Long extraWaitTimeSeconds) {
            this.maxTimeMin = maxTimeMin == null ? new WaitForReady().getMaxTimeMin() : maxTimeMin;
            this.skipFinalManifestCheck = skipFinalManifestCheck == null ? new WaitForReady().getSkipFinalManifestCheck() : skipFinalManifestCheck;
            this.requireNodesInManifest = requireNodesInManifest == null ? new WaitForReady().getRequireNodesInManifest() : requireNodesInManifest;
            this.extraWaitTimeSeconds = extraWaitTimeSeconds == null ? new WaitForReady().getExtraWaitTimeSeconds() : extraWaitTimeSeconds;
        }

        @JsonProperty
        @ApiTestConfigProperty(description="Skip the detailed manifest test in the \"checkManifest\" step? If skipped, very basic testing of the manifest is still done.")
        public boolean getSkipFinalManifestCheck() {
            return skipFinalManifestCheck;
        }


        @JsonProperty
        public long getMaxTimeMin() {
            return maxTimeMin;
        }

        @JsonProperty
        public boolean getRequireNodesInManifest() {
            return requireNodesInManifest;
        }

        @JsonProperty
        public long getExtraWaitTimeSeconds() {
            return extraWaitTimeSeconds;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof WaitForReady)) return false;

            WaitForReady that = (WaitForReady) o;

            if (maxTimeMin != that.maxTimeMin) return false;
            if (skipFinalManifestCheck != that.skipFinalManifestCheck) return false;
            if (requireNodesInManifest != that.requireNodesInManifest) return false;
            return extraWaitTimeSeconds == that.extraWaitTimeSeconds;
        }

        @Override
        public int hashCode() {
            int result = (int) (maxTimeMin ^ (maxTimeMin >>> 32));
            result = 31 * result + (skipFinalManifestCheck ? 1 : 0);
            result = 31 * result + (requireNodesInManifest ? 1 : 0);
            result = 31 * result + (int) (extraWaitTimeSeconds ^ (extraWaitTimeSeconds >>> 32));
            return result;
        }
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class Cleanup {
        private final boolean disable;
        private final boolean ignoreErrors;

        public Cleanup() {
            this.disable = false;
            this.ignoreErrors = false;
        }

        @JsonCreator
        public Cleanup(@Nullable @JsonProperty("disable") Boolean disable,
                       @Nullable @JsonProperty("ignoreErrors") Boolean ignoreErrors) {
            this.disable = disable == null ? new Cleanup().getDisable() : disable;
            this.ignoreErrors = ignoreErrors == null ? new Cleanup().getIgnoreErrors() : ignoreErrors;
        }

        @JsonProperty
        public boolean getDisable() {
            return disable;
        }

        @JsonProperty
        public boolean getIgnoreErrors() {
            return ignoreErrors;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof Cleanup)) return false;

            Cleanup cleanup = (Cleanup) o;

            if (disable != cleanup.disable) return false;
            return ignoreErrors == cleanup.ignoreErrors;
        }

        @Override
        public int hashCode() {
            int result = (disable ? 1 : 0);
            result = 31 * result + (ignoreErrors ? 1 : 0);
            return result;
        }
    }

    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    @JsonIgnoreProperties(value={"@context"})
    public static class Resource {
        @Nonnull private final Slice slice;
        @Nonnull private final RequestRSpec requestRSpec;
        @Nonnull private final Provision provision;
        @Nonnull private final WaitForReady waitForReady;
        @Nonnull private final Cleanup cleanup;

        public Resource() {
            this.slice = new Slice();
            this.requestRSpec = new RequestRSpec();
            this.provision = new Provision();
            this.waitForReady = new WaitForReady();
            this.cleanup = new Cleanup();
        }
        @JsonCreator
        public Resource(@Nullable @JsonProperty("slice") Slice slice,
                        @Nullable @JsonProperty("requestRSpec") RequestRSpec requestRSpec,
                        @Nullable @JsonProperty("provision") Provision provision,
                        @Nullable @JsonProperty("waitForReady") WaitForReady waitForReady,
                        @Nullable @JsonProperty("cleanup") Cleanup cleanup) {
            this.slice = slice == null ? new Resource().getSlice() : slice;
            this.requestRSpec = requestRSpec == null ? new Resource().getRequestRSpec() : requestRSpec;
            this.provision = provision == null ? new Resource().getProvision() : provision;
            this.waitForReady = waitForReady == null ? new Resource().getWaitForReady() : waitForReady;
            this.cleanup = cleanup == null ? new Resource().getCleanup() : cleanup;
        }

        @JsonProperty
        @Nonnull
        public Slice getSlice() {
            return slice;
        }

        @JsonProperty
        @Nonnull
        public RequestRSpec getRequestRSpec() {
            return requestRSpec;
        }

        @JsonProperty
        @Nonnull
        public Provision getProvision() {
            return provision;
        }

        @JsonProperty
        @Nonnull
        public WaitForReady getWaitForReady() {
            return waitForReady;
        }

        @JsonProperty
        @Nonnull
        public Cleanup getCleanup() {
            return cleanup;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof Resource)) return false;

            Resource resource = (Resource) o;

            if (!slice.equals(resource.slice)) return false;
            if (!requestRSpec.equals(resource.requestRSpec)) return false;
            if (!provision.equals(resource.provision)) return false;
            if (!waitForReady.equals(resource.waitForReady)) return false;
            return cleanup.equals(resource.cleanup);
        }

        @Override
        public int hashCode() {
            int result = slice.hashCode();
            result = 31 * result + requestRSpec.hashCode();
            result = 31 * result + provision.hashCode();
            result = 31 * result + waitForReady.hashCode();
            result = 31 * result + cleanup.hashCode();
            return result;
        }
    }
}
