package be.iminds.ilabt.jfed.testing.base;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;

/**
 * ApiTestConfigEditor
 */
public interface ApiTestConfigEditor<SpecificApiTestConfig extends ApiTestConfig> {
    class ApiTestConfigParameter {
        @Nonnull private final List<String> name; //"nested", example: "ansible", "enable"
        @Nullable private final String description;
        @Nonnull private final Class type;
        @Nullable private final Object defaultValue;
        private final boolean allowNull;
        @Nullable private final String editorHint;

        public ApiTestConfigParameter(@Nonnull List<String> name,
                                      @Nullable String description,
                                      @Nonnull Class type,
                                      @Nullable Object defaultValue,
                                      boolean allowNull,
                                      @Nullable String editorHint) {
            this.name = name;
            this.description = description;
            this.type = type;
            this.defaultValue = defaultValue;
            this.allowNull = allowNull;
            this.editorHint = editorHint;
        }

        @Nonnull
        public List<String> getName() {
            return name;
        }
        @Nonnull
        public String getLastName() {
            assert !name.isEmpty();
            return name.get(name.size()-1);
        }

        @Nullable
        public String getDescription() {
            return description;
        }

        @Nonnull
        public Class getType() {
            return type;
        }

        @Nullable
        public Object getDefaultValue() {
            return defaultValue;
        }

        public boolean isAllowNull() {
            return allowNull;
        }

        @Nullable
        public String getEditorHint() {
            return editorHint;
        }

        @Override
        public String toString() {
            return "ApiTestConfigParameter{" +
                    "name=" + name +
                    ", description='" + description + '\'' +
                    ", type=" + type +
                    ", defaultValue=" + defaultValue +
                    ", allowNull=" + allowNull +
                    ", editorHint='" + editorHint + '\'' +
                    '}';
        }
    }

    @Nonnull List<ApiTestConfigParameter> getParameters();
    void setParameter(@Nonnull ApiTestConfigEditor.ApiTestConfigParameter parameter, @Nullable Object value);
    @Nullable Object getParameter(@Nonnull ApiTestConfigParameter parameter);
    void setConfig(@Nonnull SpecificApiTestConfig apiTestConfig);
    @Nonnull SpecificApiTestConfig build();

    //parameters description list
    // each has:
    //   nest (list of keys in map, excluding last)   (ex:  for ansible.enable this is ["ansible"])
    //   name (last key in nested map)                (ex:  for ansible.enable this is "enable")
    //   description
    //   type: json type, so: map, list, double, int, boolean, string
    //   defaultValue
    //   allowNull: (  So: mandatory == (default == null && !allowNull)  )
    //   editorHint (optional):  multiline-5  multiline-json-20  multiline-xml-10    stringlist   urn   ...
}
